package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.util.HRDateTimeUtils;
import kd.hrmp.hbjm.common.constants.PublishConstants;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

public class HBJMMsgRepository {
    private static class HBJMMsgInstance {
        private static HBJMMsgRepository INSTANCE = new HBJMMsgRepository();
    }

    public static HBJMMsgRepository getInstance() {
        return HBJMMsgInstance.INSTANCE;
    }

    /**
     * 批量保存
     *
     * @param dynamicObjects
     *            要保存的数据
     */
    public void saveBatch(List<DynamicObject> dynamicObjects) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(PublishConstants.JOBCHANGEMSG_PAGE);
        serviceHelper.save(dynamicObjects.toArray(new DynamicObject[0]));
    }

    /**
     * 待重试的数据
     *
     * @param ids
     * @return
     */
    public DynamicObject[] queryRetryMsgsById(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(PublishConstants.JOBCHANGEMSG_PAGE);
        QFilter sendstateFilter = new QFilter("sendstate", QFilter.not_equals, PublishConstants.PUBLISH_SUCCESS);
        QFilter idsFilter = new QFilter("id", QFilter.in, ids);
        return serviceHelper.loadDynamicObjectArray(new QFilter[] {sendstateFilter, idsFilter});
    }

    public DynamicObject queryMsgById(Long id) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(PublishConstants.JOBCHANGEMSG_PAGE);
        QFilter idsFilter = new QFilter("id", QFilter.equals, id);
        return serviceHelper.queryOne("id, number, name", new QFilter[] {idsFilter});
    }

    /**
     * 待同步消费状态的数据 发送成功，并且分录状态中有失败和待更新的
     *
     * @return
     */
    public DynamicObject[] querySyncData() {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(PublishConstants.JOBCHANGEMSG_PAGE);
        Date lastSendTime = HRDateTimeUtils.addDay(new Date(), -14);
        QFilter lastSendTimeFilter = new QFilter("lastSendTime", QFilter.large_than, lastSendTime);
        QFilter sendstateFilter = new QFilter("sendstate", QFilter.equals, PublishConstants.PUBLISH_SUCCESS);
        QFilter consumeFilter =
                new QFilter("entryentity.recstatus", QFilter.not_equals, PublishConstants.RECSTATUS_SUCCESS);
        DynamicObject[] dyns =
                serviceHelper.queryOriginalArray("id", new QFilter[] {sendstateFilter, consumeFilter, lastSendTimeFilter});
        if (dyns.length == 0) {
            return null;
        }
        // 先查id，再查出所有数据。以分录的字段作为查询条件，只能查出满足条件的分录数据，后续进行保存时会导致部分分录数据丢失。
        List<Long> ids =
                Arrays.stream(dyns).map(dynamicObject -> dynamicObject.getLong("id")).collect(Collectors.toList());
        return serviceHelper.loadDynamicObjectArray(new QFilter[] {new QFilter("id", QFilter.in, ids)});
    }

    /**
     * 待同步消费状态的数据 发送成功，并且分录状态中有失败和待更新的
     *
     * @return
     */
    public int delByJobIds(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(PublishConstants.JOBCHANGEMSG_PAGE);
        QFilter idsFilter = new QFilter("job", QFilter.in, ids);
        return serviceHelper.deleteByFilter(new QFilter[] {idsFilter});
    }

    /**
     * 推送状态为 待推送的
     *
     * @return
     */
    public DynamicObject[] querySendstateSyncData() {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(PublishConstants.JOBCHANGEMSG_PAGE);
        Date lastSendTime = HRDateTimeUtils.addDay(new Date(), -3);
        QFilter lastSendTimeFilter = new QFilter("lastSendTime", QFilter.large_than, lastSendTime);
        lastSendTimeFilter.or(new QFilter("lastSendTime", QFilter.is_null, null));
        QFilter sendstateFilter = new QFilter("sendstate", QFilter.equals, PublishConstants.PUBLISH_WAIT);
        return serviceHelper.loadDynamicObjectArray(new QFilter[] {sendstateFilter, lastSendTimeFilter});
    }
}
