package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.basedata.service.BaseDataServiceImpl;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.permission.api.HasPermOrgResult;
import kd.bos.servicehelper.basedata.BaseDataServiceHelper;
import kd.bos.servicehelper.model.PermissionStatus;
import kd.bos.servicehelper.permission.PermissionServiceHelper;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.HRHisBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hr.hbp.common.util.HRDateTimeUtils;
import kd.hr.hbp.common.util.HRObjectUtils;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 职位类数据库操作类
 *
 * @author rimwood
 * @since 2021-12-08 17:37:42
 */
public class JobClassRepository {
    /**
     * 日志
     */
    private static final Log logger = LogFactory.getLog(JobClassRepository.class);

    private static class JobClassInstance {
        private static JobClassRepository INSTANCE = new JobClassRepository();
    }

    public static JobClassRepository getInstance() {
        return JobClassInstance.INSTANCE;
    }

    /**
     * 查询受控业务单元下的职位类
     * @param orgId 受控业务单元id
     * @return 职位类数组
     */
    public DynamicObject[] queryJobClassByOrgId(Long orgId){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter orgFilter = BaseDataServiceHelper.getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBCLASS, orgId);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.not_in,
                new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED});
        return serviceHelper.query("id,name,number,parent,jobclasslevel,jobfamily,jobseq",
                new QFilter[]{currentFilter,orgFilter,enableFilter,initStatusFilter},"number asc,id desc");
    }

    /**
     * 根据职位类id集合查询职位族
     * @param ids 职位类id集合
     * @return 职位类数组
     */
    public DynamicObject[] queryJobClassByIds(List<Long> ids){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter idsFilter = new QFilter(HRBaseConstants.ID, QFilter.in, ids);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.query("id,name,number,parent,jobclasslevel,jobfamily,jobseq,createorg,enable",new QFilter[]{currentFilter,idsFilter,enableFilter},"number asc,id desc");
    }

    /**
     * 根据职位类id集合查询职位族,忽略状态
     * @param ids 职位类id集合
     * @return 职位类数组
     */
    public DynamicObject[] queryJobClassByIdsIgnoreEnable(List<Long> ids){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter idsFilter = new QFilter(HRBaseConstants.ID, QFilter.in, ids);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.query("id,name,number,parent,jobclasslevel,jobfamily,jobseq,createorg,enable",new QFilter[]{currentFilter,idsFilter},"number asc,id desc");
    }

    /**
     * 根据职位族id集合查询职位类
     * @param ids 职位类id集合
     * @return 职位类数组
     */
    public DynamicObject[] queryJobFamilyByIds(Collection<Long> ids){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter idsFilter = new QFilter("jobfamily", QFilter.in, ids);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.queryOriginalArray("jobfamily",new QFilter[]{currentFilter,idsFilter,enableFilter});
    }

    /**
     * 根据职位类id集合查询职位子类
     * @param ids 职位类id集合
     * @return 职位类数组
     */
    public DynamicObject[] querySubJobClassByIds(Collection<Long> ids){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter idsFilter = new QFilter("parent", QFilter.in, ids);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.queryOriginalArray("id, number, name, parent",new QFilter[]{currentFilter,idsFilter,enableFilter});
    }

    /**
     * 根据用户id查询对应权限下职位类id
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobClass(Long userId, Boolean isShowDisable) {
        QFilter enableFilter = new QFilter("1", QCP.equals,1);
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        if (!Objects.isNull(getClassFilter(userId, isShowDisable))) {
            enableFilter.and(getClassFilter(userId, isShowDisable));
        }
        return serviceHelper.queryOriginalArray(HRBaseConstants.ID, new QFilter[]{enableFilter});
    }

    /**
     * 根据用户id查询对应权限下职位类过滤
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public QFilter getClassFilter(Long userId, Boolean isShowDisable) {
        // 权限查询
        QFilter permFilter = new QFilter("1",QCP.equals,1);
        HasPermOrgResult orgPerm = PermissionServiceHelper.getAllPermOrgs(userId, HBJMConstants.APP_HJM,
                HBJMConstants.PAGE_HBJM_JOBCLASS, PermissionStatus.View);
        if (!Objects.isNull(orgPerm) && !orgPerm.hasAllOrgPerm()) {
            logger.info("JobClassQueryRepository.queryJobClass():do not has all orgPermission");
            List<Long> perms = orgPerm.getHasPermOrgs();
            if (Objects.isNull(perms) || perms.isEmpty()) {
                logger.info("JobClassQueryRepository.queryJobClass():orgPermission is empty");
                return null;
            }
            QFilter qFilter = new BaseDataServiceImpl().getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBCLASS, perms, true);
            permFilter.or(qFilter);

        }
        return permFilter;
    }

    /**
     * 根据用户id查询对应权限下职位族id
     *
     * @param userId        用户id
     * @param isShowDisable 是否显示禁用
     * @return dys
     */
    public DynamicObject[] queryJobFamily(Long userId, Boolean isShowDisable) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBFAMILIY);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.not_equals, HRBaseConstants.ENABLING);
        } else {
            enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        }
        HasPermOrgResult orgPerm = PermissionServiceHelper.getAllPermOrgs(userId, HBJMConstants.APP_HJM,
                HBJMConstants.PAGE_HBJM_JOBFAMILIY, PermissionStatus.View);
        if (!Objects.isNull(orgPerm) && !orgPerm.hasAllOrgPerm()) {
            logger.info("JobClassQueryRepository.queryJobFam():do not has all orgPermission");
            List<Long> perms = orgPerm.getHasPermOrgs();
            if (Objects.isNull(perms) || perms.isEmpty()) {
                logger.info("JobClassQueryRepository.queryJobFam():orgPermission is empty");
                return null;
            }
            QFilter permFilter = new BaseDataServiceImpl().getBaseDataFilter(HBJMConstants.PAGE_HBJM_JOBFAMILIY, perms, true);
            enableFilter.and(permFilter);
        }
        return serviceHelper.queryOriginalArray(HRBaseConstants.ID, new QFilter[]{enableFilter});
    }

    /**
     * 根据职位序列id查询职位类id
     *
     * @param seqId 职位序列id
     * @param filedId 字段id
     * @param field 字段
     * @param isShowDisable 是否显示禁用
     * @return
     */
    public List<Long> queryJobId(Long seqId, List<Long> filedId, String field, Boolean isShowDisable) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter filter = new QFilter(field, QFilter.in, filedId);
        if (Objects.nonNull(getClassFilter(Long.valueOf(RequestContext.get().getUserId()),isShowDisable))) {
            filter.and(getClassFilter(Long.valueOf(RequestContext.get().getUserId()),isShowDisable));
        }
        DynamicObject[] dynamicObjects = serviceHelper.queryOriginalArray(HRBaseConstants.ID, new QFilter[]{ filter});
        if (HRObjectUtils.isEmpty(dynamicObjects) || dynamicObjects.length == 0) {
            return new ArrayList<>();
        }
        return Arrays.stream(dynamicObjects).map(dy -> dy.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
    }

    /**
     * 根据职位类boid查询历史版本
     * @param boidList
     * @return
     */
    public DynamicObject[] getJobClassHisByBoId(Long[] boidList) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter boidQFilter = new QFilter(HisFieldNameConstants.BOID, QCP.in, boidList);
        QFilter iscurrentversionQfilter = new QFilter(JobConstans.IS_CURRENT_VERSION, QCP.equals, "0");
        QFilter datastatusQFilter = new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QCP.in,
                new String[]{HRHisBaseConstants.EFFECTSTATUS_SCHEDULEEFFECT, HRHisBaseConstants.EFFECTSTATUS_EFFECTING,
                        HRHisBaseConstants.EFFECTSTATUS_INVALID});
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.equals, HBJMConstants.INIT_STATUS_COMPLETE);
        QFilter qFilter = boidQFilter.and(datastatusQFilter).and(iscurrentversionQfilter).and(initStatusFilter);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{qFilter});
    }

    public DynamicObject[] getJobClassHisByFamilyBoId(List<Long> jobFamilyIdList, String selectFields) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBCLASS);
        QFilter boidQFilter = new QFilter("jobfamily.id", QCP.in, jobFamilyIdList);
        QFilter iscurrentversionQfilter = new QFilter(JobConstans.IS_CURRENT_VERSION, QCP.equals, "1");
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.equals, HBJMConstants.INIT_STATUS_COMPLETE);
        QFilter qFilter = boidQFilter.and(iscurrentversionQfilter).and(initStatusFilter);
        return serviceHelper.query(selectFields, new QFilter[]{qFilter});
    }
}
