package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.HRHisBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hrmp.hbjm.business.utils.QFilterCreatorUtils;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.List;
import java.util.Set;

/**
 * @author rd_xiangquan_zheng
 * @description 职级方案下数据库操作Repository
 * @since 2022/3/1 15:36
 */
public class JobLevelScmRepository {
    private static class JobLevelScmInstance {
        private static JobLevelScmRepository INSTANCE = new JobLevelScmRepository();
    }

    public static JobLevelScmRepository getInstance() {
        return JobLevelScmRepository.JobLevelScmInstance.INSTANCE;
    }

    /**
     * 根据职级方案id集合查询职等集合
     *
     * @param rowIds 职级方案rowIds
     * @return dyns 职级集合
     */
    public DynamicObject[] queryJobLevelScmByrowIds(List<Long> rowIds) {
        HRBaseServiceHelper hrBaseServiceHelper = new HRBaseServiceHelper("hbjm_joblevelscmhr");
        QFilter joblevelFilter = new QFilter("id", QFilter.in, rowIds);
        return hrBaseServiceHelper.query("id, boid, entryentity.entryboid, entryentity.joblevel_number, entryentity.joblevel_name, entryentity.joblevel_seq",
                new QFilter[]{joblevelFilter});
    }

    /**
     * 根据职位dyn查询职级及职级方案信息
     *
     * @param levelscmids 职位动态对象
     * @return 职级
     */
    public DynamicObject[] getLevelScms(Set<Long> levelscmids) {
        HRBaseServiceHelper scmlevelserviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBLEVELSCM);
        QFilter levelidFilter = new QFilter(HRBaseConstants.ID, QFilter.in, levelscmids);
        return scmlevelserviceHelper.query("entryentity,entryentity.joblevel_number,entryentity.joblevel_name,entryentity.joblevel_seq", new QFilter[]{levelidFilter});
    }

    /**
     * 根据职等ids查询职等
     * @param levelIds 职等ids
     * @return 职等
     */
    public DynamicObject[] getJobLevelById(Set<Long> levelIds) {
        HRBaseServiceHelper scmgradeserviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBLEVEL);
        QFilter gradeidFilter = new QFilter(HRBaseConstants.ID, QFilter.in, levelIds);
        return scmgradeserviceHelper.loadDynamicObjectArray(new QFilter[]{gradeidFilter});
    }

    /**
     * 根据职级方案number查询职级及职级方案信息
     *
     * @param levelScmNumberSet 职等方案number Set
     * @return 职等
     */
    public DynamicObject[] getLevelScmDynsByNumberSet(Set<String> levelScmNumberSet) {
        HRBaseServiceHelper levelScmServiceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBLEVELSCM);
        QFilter levelNumberFilter = new QFilter(HRBaseConstants.NUMBER, QFilter.in, levelScmNumberSet);
        return levelScmServiceHelper.query("id, number, boid, createorg", new QFilter[]{levelNumberFilter, QFilterCreatorUtils.createCurrentVersionFilter()});
    }

    /**
     * 根据职位方案boid查询历史版本
     * @param boidList
     * @return
     */
    public DynamicObject[] getJobLevelScmByBoId(Long[] boidList) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBLEVELSCM);
        QFilter boidQFilter = new QFilter(HisFieldNameConstants.BOID, QCP.in, boidList);
        QFilter iscurrentversionQfilter = new QFilter(JobConstans.IS_CURRENT_VERSION, QCP.equals, "0");
        QFilter datastatusQFilter = new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QCP.in,
                new String[]{HRHisBaseConstants.EFFECTSTATUS_SCHEDULEEFFECT, HRHisBaseConstants.EFFECTSTATUS_EFFECTING,
                        HRHisBaseConstants.EFFECTSTATUS_INVALID});
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.equals, HBJMConstants.INIT_STATUS_COMPLETE);
        QFilter qFilter = boidQFilter.and(datastatusQFilter).and(iscurrentversionQfilter).and(initStatusFilter);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{qFilter});
    }

    /**
     * 根据id查询权限范围内的数据
     * @param idList
     * @param permFilter
     * @return
     */
    public DynamicObject[] getJobLevelScmById(Set<Long> idList,QFilter permFilter) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBLEVELSCM);
        QFilter idFilter = new QFilter(HRBaseConstants.ID, QFilter.in, idList);
        return serviceHelper.query("id, number, boid", new QFilter[]{idFilter,permFilter});
    }
}
