package kd.hrmp.hbjm.business.domain.service.impl;

import kd.bos.basedata.service.BaseDataServiceImpl;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.db.tx.TX;
import kd.bos.db.tx.TXHandle;
import kd.hr.hbp.business.log.ModifyDirtyManager;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hrmp.hbjm.business.domain.repository.JobClassScmRepository;
import kd.hrmp.hbjm.business.domain.repository.JobFamilyScmRepository;
import kd.hrmp.hbjm.business.domain.repository.JobScmRepository;
import kd.hrmp.hbjm.business.domain.repository.JobSeqScmRepository;
import kd.hrmp.hbjm.business.domain.service.JobScmService;

import java.util.ArrayList;
import java.util.List;

public class JobScmServiceImpl implements JobScmService {
    @Override
    public boolean saveAll(DynamicObject jobScmBaseInfo, DynamicObjectCollection jobSeqScmCollection, DynamicObjectCollection jobFamilyScmCollection, DynamicObjectCollection jobClassScmCollection) {
        boolean result = false;
        // 中台日志
        ModifyDirtyManager modifyDirtyManager = new ModifyDirtyManager();
        DynamicObject[] dys = new DynamicObject[]{jobScmBaseInfo};
        try (TXHandle txHandle = TX.required()) {
            try {
                // 日志操作
                modifyDirtyManager.init(dys[HRBaseConstants.INT_ZERO].getDynamicObjectType(),dys,"save");
                // 保存基本信息
                JobScmRepository.getInstance().saveBaseInfo(jobScmBaseInfo);
                // 保存关联的职位序列
                JobSeqScmRepository.getInstance().saveBatch(jobSeqScmCollection);
                // 保存关联的职位族
                JobFamilyScmRepository.getInstance().saveBatch(jobFamilyScmCollection);
                // 保存关联的职位类
                JobClassScmRepository.getInstance().saveBatch(jobClassScmCollection);
                result = true;
            } catch (Exception ex) {
                // 回滚平台数据
                txHandle.markRollback();
                return false;
            }
        }
        // 存日志
        if(result){
            modifyDirtyManager.batchInsertLog(dys);
        }
        // 更新_U表
        List ids = new ArrayList<Long>();
        ids.add(jobScmBaseInfo.getLong("org_id"));
        new BaseDataServiceImpl().refreshBaseDataUseRange("hbjm_jobscmhr", ids);
        return result;
    }

    @Override
    public boolean updateAll(DynamicObject jobScmBaseInfo, DynamicObjectCollection jobSeqScmCollection,DynamicObjectCollection jobFamilyScmCollection,DynamicObjectCollection jobClassScmCollection) {
        boolean result = false;
        ModifyDirtyManager modifyDirtyManager = new ModifyDirtyManager();
        DynamicObject[] dys = new DynamicObject[]{jobScmBaseInfo};
        try (TXHandle txHandle = TX.required()) {
            try {
                // 日志操作
                modifyDirtyManager.init(dys[HRBaseConstants.INT_ZERO].getDynamicObjectType(),dys,"save");
                // 保存基本信息
                JobScmRepository.getInstance().updateBaseInfo(jobScmBaseInfo);
                // 先删除关联的职位类
                JobClassScmRepository.getInstance().deleteByScmId(jobScmBaseInfo.getLong(HRBaseConstants.ID));
                // 先删除关联的职位族
                JobFamilyScmRepository.getInstance().deleteByScmId(jobScmBaseInfo.getLong(HRBaseConstants.ID));
                // 先删除关联的职位序列
                JobSeqScmRepository.getInstance().deleteByScmId(jobScmBaseInfo.getLong(HRBaseConstants.ID));
                // 保存关联的职位序列
                JobSeqScmRepository.getInstance().saveBatch(jobSeqScmCollection);
                // 保存关联的职位族
                JobFamilyScmRepository.getInstance().saveBatch(jobFamilyScmCollection);
                // 保存关联的职位类
                JobClassScmRepository.getInstance().saveBatch(jobClassScmCollection);
                result = true;
            } catch (Exception ex) {
                // 回滚平台数据
                txHandle.markRollback();
                return false;
            }
        }
        // 存日志
        if(result){
            modifyDirtyManager.batchInsertLog(dys);
        }
        // 更新_U表
        List ids = new ArrayList<Long>();
        ids.add(jobScmBaseInfo.getLong("org_id"));
        new BaseDataServiceImpl().refreshBaseDataUseRange("hbjm_jobscmhr", ids);
        return result;
    }

}
