package kd.hrmp.hbjm.business.init;

import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.entity.operate.result.IOperateInfo;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.property.FieldProp;
import kd.bos.exception.KDBizException;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.service.operation.OperationServiceImpl;
import kd.hr.hbp.business.init.AbstractInitDomainDataService;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hrmp.hbjm.common.constants.HBJMConstants;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 带历史的职位基础资料初始化处理服务
 *
 * @author rd_ningjie
 * @since 2022年03月12日 12:33:06
 */
public class HisBaseDataComsumerService extends AbstractInitDomainDataService {

    private static Log LOG = LogFactory.getLog(HisBaseDataComsumerService.class);

    @Override
    public void validate() {
        //基础资料校验也走save op，会在op中做处理，校验会cancel保存操作
        this.validateAndSave(HBJMConstants.INIT_TYPE_SAVE_VALIDATE, HBJMConstants.OP_SAVE);
    }

    @Override
    public void save() {
        this.validateAndSave(HBJMConstants.INIT_TYPE_SAVE, HBJMConstants.OP_SAVE);
    }

    void validateAndSave(String type, String op) {

        LOG.info("HisBaseDataComsumerService.validateAndSave start:{}", type, op);

        String entityCode = this.getInitInParam().getEntityNumber();
        Map<String, Object> map = this.getInfoWithDynamicObject();
        List<DynamicObject> list = (List) map.get("data");
        List<Map<String, Object>> datamap = (List) this.getInfoWithMap().get("data");

        Map<Long, Set<Long>> idMapping = (Map<Long, Set<Long>>) map.get("idmapping");
        //如果没有处理数据，则直接返回，打印日志
        if (ObjectUtils.isEmpty(list) || ObjectUtils.isEmpty(idMapping) || ObjectUtils.isEmpty(datamap)) {
            LOG.info("HisBaseDataComsumerService.validateAndSave:{}", "init data is null");
            return;
        }

        //数据自定义处理，业务可以重写，对数据进行处理
        this.dealDynData(list,datamap);

        //设置初始化字段值
        setInitDefalutValue(list);

        //默认是校验
        type = ObjectUtils.isEmpty(type) ? HBJMConstants.INIT_TYPE_SAVE_VALIDATE : type;
        //如果是保存操作则先删除数据
        if (HBJMConstants.INIT_TYPE_SAVE.equals(type)) {
            HRBaseServiceHelper hrBaseServiceHelper = new HRBaseServiceHelper(entityCode);
            //根据主键删除数据，再走op新增操作
            hrBaseServiceHelper.delete(idMapping.keySet().toArray());
        }

        OperationResult opResult = callOp(op, list.toArray(new DynamicObject[0]), type);
        if(!opResult.isSuccess() && opResult.getBillCount() <=0){
            LOG.info("HisBaseDataComsumerService.validateAndSave call op fail result:{}", opResult);
            throw new KDBizException(opResult.getMessage());
        }
        List<IOperateInfo> infos = opResult.getAllErrorOrValidateInfo();
        Map<Object, String> returnInfo = new HashMap<>();
        if (!opResult.isSuccess() && !ObjectUtils.isEmpty(infos)) {
            infos.stream().forEach(info -> {
                Long kk = (Long) info.getPkValue();
                String valStr = returnInfo.get(kk);
                if (valStr != null) {
                    returnInfo.put(kk, valStr + "\n" + info.getMessage());
                } else {
                    returnInfo.put(kk, info.getMessage());
                }
            });
        }

        //处理错误信息返回
        idMapping.forEach((key, val) -> {
            if (!ObjectUtils.isEmpty(val)) {
                if (returnInfo.containsKey(key)) {
                    val.forEach(id -> {
                        this.getInitOutParam().addErrorMsg(id, returnInfo.get(key));
                    });
                } else {
                    val.forEach(id -> {
                        this.getInitOutParam().addSuccessMsg(id, key.toString(), "");
                    });
                }
            }
        });
        LOG.info("HisBaseDataComsumerService.validateAndSave end:{}", this.getInitOutParam());
    }

    private static OperationResult callOp(String opKey, DynamicObject[] dyns, String type) {
        OperationServiceImpl opImpl = new OperationServiceImpl();
        OperateOption operateOption = OperateOption.create();
        operateOption.setVariableValue("initType", type);
        return opImpl.localInvokeOperation(opKey, dyns, operateOption);
    }

    //处理自定义数据,供子类重写
    protected void dealDynData(List<DynamicObject> dyns, List<Map<String, Object>> datamaps) {

        if (!ObjectUtils.isEmpty(dyns)) {
            DataEntityPropertyCollection pros = dyns.get(0).getDataEntityType().getProperties();
            dyns.stream().forEach(dyn -> {
                pros.forEach(pro -> {
                    if (pro instanceof FieldProp) {
                        FieldProp pp = (FieldProp) pro;
                        //如果是必填字段，而且有默认值，并且没有设置值，则赋予默认值
                        if (pp.isMustInput() && !ObjectUtils.isEmpty(pp.getDefValue())
                                && ObjectUtils.isEmpty(dyn.get(pro.getName()))) {
                            dyn.set(pro.getName(), pp.getDefValue());
                        }
                    }
                });
            });
        }
        return;
    }
}
