package kd.hrmp.hbjm.business.task;

import kd.bos.context.RequestContext;
import kd.bos.db.tx.TX;
import kd.bos.db.tx.TXHandle;
import kd.bos.exception.KDException;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.schedule.api.ScheduleManager;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.service.ServiceFactory;
import kd.hrmp.hbjm.business.upgrade.impl.JobEventDataUpgradeServiceImpl;

import java.util.Map;


/**
 * 调度计划 hjm_dataUpGradeTask_SKDP_S 执行的定时任务
 */
public class DataUpgradeTask extends AbstractTask {
    private static final Log LOG = LogFactory.getLog(DataUpgradeTask.class);

    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        long startTime = System.currentTimeMillis();
        LOG.info("数据升级任务 开始执行:");
        executeUpgrade();
        disableTask();
        LOG.info("数据升级任务: 执行完，共耗时 {} 毫秒", System.currentTimeMillis() - startTime);
    }


    /**
     * 不能同时写多个库, 各自开启新的事务
     */
    private void executeUpgrade() {
        try (TXHandle required = TX.required()) {
            try {
                // 调用 职位事数据升级服务
                new JobEventDataUpgradeServiceImpl().jobEventDataUpgrade();
                new HbjmUpdateDataTask().afterExecuteSqlWithResult("", "", "hr", "");
            } catch (Exception ex) {
                required.markRollback();
                LOG.error("staff upgrade error", ex);
            }
        }
    }

    private void disableTask(){
		ScheduleManager scheduleManager = ServiceFactory.getService(ScheduleManager.class);
		// 禁用调度作业id
		scheduleManager.disableJob("3=U+3M4M58=P");
		// 禁用调度计划id
		scheduleManager.disableSchedule("3=U+EVU85UKA");
	}
}
