package kd.hrmp.hbjm.business.utils;

import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.entity.tree.TreeNode;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

/**
 * @author rd_ningjie
 * @description:职位、职位类树构建工具类
 * @date 2022/7/18 15:10
 */
public class JobTreeBuildUtil {


    public static TreeNode appendNode(Map<String, TreeNode> treeNodes, String parentId, String currentId, Function<TreeNode, TreeNode> func) {
        TreeNode parentNode = treeNodes.get(parentId);
        if (ObjectUtils.isEmpty(parentNode)) {
            return null;
        }
        //如果当前节点已经被构造过，则跳过
        TreeNode currentNode = treeNodes.get(currentId);
        if (!ObjectUtils.isEmpty(currentNode)) {
            return null;
        }
        currentNode = func.apply(parentNode);
        addChildTreeNode(parentNode, currentNode);
        treeNodes.put(currentId, currentNode);
        return currentNode;
    }


    /**
     * @param parentNode
     * @param childNode
     * @description: 添加子节点
     * @return: kd.bos.entity.tree.TreeNode
     * @author rd_ningjie
     * @date: 2022/6/30 14:33
     */
    public static TreeNode addChildTreeNode(TreeNode parentNode, TreeNode childNode) {
        if (!ObjectUtils.isEmpty(parentNode) && !ObjectUtils.isEmpty(childNode)) {
            List<TreeNode> list = parentNode.getChildren();
            list = list == null ? new ArrayList<>() : list;
            list.add(childNode);
            parentNode.setChildren(list);
        }
        return parentNode;
    }

}
